@@ -1,5 +1,6 @@
 /*
- * Copyright 2015, 2016 Andrew Ayer
+ * Copyright 2015, 2016 Andrew Ayer <agwa@andrewayer.name>
+ * Copyright 2016, 2017 Chris Lamb <lamby@debian.org>
  *
  * This file is part of disorderfs.
  *
@@ -42,7 +43,7 @@ extern "C" {
 #include <sys/file.h>
 #include <stddef.h>
 
-#define DISORDERFS_VERSION "0.5.1"
+#define DISORDERFS_VERSION "0.5.3"
 
 namespace {
 	std::vector<std::string>	bare_arguments;
@@ -56,6 +57,7 @@ namespace {
 		int			sort_dirents{0};
 		int			pad_blocks{1};
 		int			share_locks{0};
+		int			quiet{0};
 	};
 	Disorderfs_config		config;
 
@@ -66,7 +68,7 @@ namespace {
 	}
 
 	int wrap (int retval) { return retval == -1 ? -errno : 0; }
-	using Dirents = std::vector<std::string>;
+	using Dirents = std::vector<std::pair<std::string, ino_t>>;
 
 	// The libc versions of seteuid, etc. set the credentials for all threads.
 	// We need to set credentials for a single thread only, so call the syscalls directly.
@@ -180,7 +182,8 @@ namespace {
 	struct fuse_operations		disorderfs_fuse_operations;
 	enum {
 		KEY_HELP,
-		KEY_VERSION
+		KEY_VERSION,
+		KEY_QUIET
 	};
 #define DISORDERFS_OPT(t, p, v) { t, offsetof(Disorderfs_config, p), v }
 	const struct fuse_opt disorderfs_fuse_opts[] = {
@@ -199,6 +202,8 @@ namespace {
 		FUSE_OPT_KEY("--help", KEY_HELP),
 		FUSE_OPT_KEY("-V", KEY_VERSION),
 		FUSE_OPT_KEY("--version", KEY_VERSION),
+		FUSE_OPT_KEY("-q", KEY_QUIET),
+		FUSE_OPT_KEY("--quiet", KEY_QUIET),
 		FUSE_OPT_END
 	};
 	int fuse_opt_proc (void* data, const char* arg, int key, struct fuse_args* outargs)
@@ -212,6 +217,7 @@ namespace {
 			std::clog << "    -o opt,[opt...]        mount options (see below)" << std::endl;
 			std::clog << "    -h, --help             display help" << std::endl;
 			std::clog << "    -V, --version          display version info" << std::endl;
+			std::clog << "    -q, --quiet            don't output any status messages" << std::endl;
 			std::clog << std::endl;
 			std::clog << "disorderfs options:" << std::endl;
 			std::clog << "    --multi-user=yes|no    allow multiple users to access overlay (requires root; default: no)" << std::endl;
@@ -229,6 +235,9 @@ namespace {
 			fuse_opt_add_arg(outargs, "--version");
 			fuse_main(outargs->argc, outargs->argv, &disorderfs_fuse_operations, nullptr);
 			std::exit(0);
+		} else if (key == KEY_QUIET) {
+			config.quiet = true;
+			return 0;
 		}
 		return 1;
 	}
@@ -261,21 +270,23 @@ int	main (int argc, char** argv)
 
 	// Add some of our own hard-coded FUSE options:
 	fuse_opt_add_arg(&fargs, "-o");
-	fuse_opt_add_arg(&fargs, "atomic_o_trunc,default_permissions"); // XXX: other mount options?
+	fuse_opt_add_arg(&fargs, "atomic_o_trunc,default_permissions,use_ino"); // XXX: other mount options?
 	if (config.multi_user) {
 		fuse_opt_add_arg(&fargs, "-o");
 		fuse_opt_add_arg(&fargs, "allow_other");
 	}
 	fuse_opt_add_arg(&fargs, bare_arguments[1].c_str());
 
-	if (config.shuffle_dirents) {
-		std::cout << "disorderfs: shuffling dirents" << std::endl;
-	}
-	if (config.reverse_dirents) {
-		std::cout << "disorderfs: reversing dirents" << std::endl;
-	}
-	if (config.sort_dirents) {
-		std::cout << "disorderfs: sorting dirents" << std::endl;
+	if (!config.quiet) {
+		if (config.shuffle_dirents) {
+			std::cout << "disorderfs: shuffling dirents" << std::endl;
+		}
+		if (config.reverse_dirents) {
+			std::cout << "disorderfs: reversing dirents" << std::endl;
+		}
+		if (config.sort_dirents) {
+			std::cout << "disorderfs: sorting dirents" << std::endl;
+		}
 	}
 
 	/*
@@ -418,7 +429,7 @@ int	main (int argc, char** argv)
 		struct dirent*	dirent_p;
 		int		res;
 		while ((res = readdir_r(d, &dirent_storage, &dirent_p)) == 0 && dirent_p) {
-			dirents->emplace_back(dirent_p->d_name);
+			dirents->emplace_back(std::make_pair(dirent_p->d_name, dirent_p->d_ino));
 		}
 		if (config.sort_dirents) {
 			std::sort(dirents->begin(), dirents->end());
@@ -436,14 +447,17 @@ int	main (int argc, char** argv)
 	};
 	disorderfs_fuse_operations.readdir = [] (const char* path, void* buf, fuse_fill_dir_t filler, off_t offset, struct fuse_file_info* info) {
 		Dirents&		dirents = *get_fuse_data<Dirents*>(info);
+		struct stat		st;
+		memset(&st, 0, sizeof(st));
 		if (config.shuffle_dirents) {
 			std::random_device	rd;
 			std::mt19937		g(rd());
 			std::shuffle(dirents.begin(), dirents.end(), g);
 		}
 
-		for (const std::string& dirent : dirents) {
-			if (filler(buf, dirent.c_str(), nullptr, 0) != 0) {
+		for (const auto dirent : dirents) {
+			st.st_ino = dirent.second;
+			if (filler(buf, dirent.first.c_str(), &st, 0) != 0) {
 				return -ENOMEM;
 			}
 		}
